
use Getopt::Long;
use POSIX;
use Socket;

use strict;
my $PAL_VERSION = "pal33";
my $BASE_DIR = "/opt/VRTSobc/$PAL_VERSION/bin";
my $REGCTL = "$BASE_DIR/vxregctl";
my $VEACONFIG = "$BASE_DIR/veaconfig";
my $VXINI = "$BASE_DIR/vxini";

my $REG_FILE = "/etc/vx/isis/Registry";
my $REG_BCK = "/var/vx/isis/reg.entries";
my $LOG = "/var/vx/isis/clusterize.log";
my $CSF_RESOLV_FILE = "/etc/default/csf_resolv.conf";
my $VCS_LOG = "/var/vx/isis/clusterize_vcs.log";
my $DBS_SCRIPT_DIR = "/opt/VRTSobc/$PAL_VERSION/install/vcs/dbserver";
my $DB_SCRIPT_DIR = "/opt/VRTSobc/$PAL_VERSION/install/vcs/db";

my $VCSAgentPath = "/opt/VRTSobc/$PAL_VERSION/install";
my %conf = ();
my %mount_conf = ();
my %sg_conf = ();
my %nic_conf =();
my %webapp_conf = ();
my %asa_database = ();
my @db_mount_agents = ();
my %volumes = ();
my %disk_groups = ();

my $asa_db_server;
my $temp  = "/var/vx/isis/clusterize.temp";
my $temp2 = "/var/vx/isis/clusterize.temp2";
my $cs_configured = 0;

my $VXDBMS_BASEDIR;
my $opts_c;
my $opts_p;
my $opts_s;
my $opts_h;

use constant INFO => "INFO";
use constant WARN => "WARNING";

use constant ERR => "ERROR";

my $error_level = "ERROR";
if(exists($ENV{'CLUSTERIZE_ERROR_LEVEL'}))
{
    $error_level = $ENV{'CLUSTERIZE_ERROR_LEVEL'};
}

my $ret = GetOptions('conf=s' => \$opts_c,'password=s' => \$opts_p,'setup' => \$opts_s,'help' => \$opts_h);

if (!$ret)
{
   Usage();
   exit(1);
     
}

my $conf_file = $opts_c;
my $passwd = $opts_p;
my $ip;

if ($opts_h)
{
    LongUsage();
    exit(0);
}

`echo Starting cluster setup >> $LOG`;

#Get the VCS_HOME nd VCS_CONF from env variables if set
#else use default locations.

my $VCS_HOME = $ENV{"VCS_HOME"};
if($VCS_HOME eq "")
{
   $VCS_HOME = "/opt/VRTSvcs";
}

if(! -d $VCS_HOME)
{
   print_msg(ERR,"VCS not installed at location $VCS_HOME.\n"
   ."Please verify if VCS is installed on this machine.\n");
   exit(1);
}

log_msg("Using VCS_HOME $VCS_HOME");

my $VCS_CONF = $ENV{"VCS_CONF"};
if($VCS_CONF eq "")
{
   $VCS_CONF = "/etc/VRTSvcs";
}

log_msg("Using VCS_CONF $VCS_CONF");

#Validate the config file
ValidateConfFile();

if ($opts_s)
{#If Master

   ParseServiceGroup();
   ParseNIC();
   ParseIP();
   ParseDiskGroups();
   ParseVolumes();
   ParseMOUNTS();
   ParseWebApp();
   ParseASADBServer();
   ParseASADatabase();
   ParseAgents();

   log_msg("Parsed all sections.Starting VCS configuration");

   open(VCSFD,">$VCS_LOG");

   #make the VCS conf file writable
   ExecuteCmd("haconf -makerw ",
   "ERROR: failed to make cluster configuration file writable.Check if VCS is up". 
   "on the machine\n");

   vcs_log("haconf -dump -makero");
   configure_service_grp();
   configure_nic();
   configure_ip();
   configure_disk_groups();
   configure_volumes();
   configure_mounts();
   configure_webapp();
   configure_asa_dbserver();
   configure_asa_database();

   vcs_log("hatype -delete VeaAgent");
   configure_vea_agent(); 

   #If CS is one of the agents in the config file then create the following dependancies
   # GC->vxsvc , GN->GC , AA->GN

   if ($cs_configured)
   {
        ExecuteCmd("hares -link $sg_conf{'GRP_NAME'}_gridcentral $sg_conf{'GRP_NAME'}_Root", 
        "ERROR: failed to create dependancy of gridcentral agent on the vxsvc agent\n");
        vcs_log("hares -unlink $sg_conf{'GRP_NAME'}_gridcentral $sg_conf{'GRP_NAME'}_Root");

        ExecuteCmd("hares -link $sg_conf{'GRP_NAME'}_actionagent $sg_conf{'GRP_NAME'}_gridcentral",
        "ERROR: failed to create dependancy of actionagent on gridcentral\n"); 
        vcs_log("hares -unlink $sg_conf{'GRP_NAME'}_actionagent $sg_conf{'GRP_NAME'}_gridcentral");

        ExecuteCmd("hares -link $sg_conf{'GRP_NAME'}_VEAWeb  $sg_conf{'GRP_NAME'}_actionagent",
        "ERROR: failed to create dependancy of VEAWeb app on the agent for vxsvc");
        vcs_log("hares -unlink $sg_conf{'GRP_NAME'}_VEAWeb $sg_conf{'GRP_NAME'}_actionagent");
   }

   ExecuteCmd("hagrp -enableresources $sg_conf{'GRP_NAME'}",
   "ERROR: Failed to enable resources in VCS\n");

   #Finally save the changes to the main.cf file
   ExecuteCmd("haconf -dump -makero","ERROR: Failed to save main.cf\n");

   #Let the dump finish
   sleep 5;

   close(VCSFD);
   log_msg("cluster setup done for $sg_conf{'GRP_NAME'} in VCS");

}
else
{
   #If not Master configuration is needed only for VEA Agent resources
   ParseServiceGroup();
   ParseASADatabase();
   ParseAgents();

   configure_asa_database();
   configure_vea_agent(); 

   log_msg("cluster setup done for the slave"); 
}
exit_success();

#End of main flow

###########Subroutines start


sub configure_vea_agent()
{
    #1.Update registry entries to point to the shared location.
    #2.Set the virtual name and persistent id for this agent

    #Incase of -s option do the following also
    #1.Copy the files to be shared to the shared location.
    #2.Rejoin the registered domains using the virtual host name and the specified persistent id.
    #3.configure a VeaAgent resource in main.cf

    for my $agent_info ( keys %conf )
    {
       my $agent_name = $conf{$agent_info}->{'NAME'};
       my $AgentRootKey;
       my $mount_point = $conf{$agent_info}->{'MOUNT_POINT'};
       my $virtual_name = $conf{$agent_info}->{'VIRTUAL_HOSTNAME'};
       my $pid = $conf{$agent_info}->{'PID'};
       my $shared_dir = ${mount_point}."/".${virtual_name};

 
       open(REGFD,">$REG_BCK");


       if ( ($agent_name eq "Root") || ($agent_name eq "root") || ($agent_name eq "CS") )
       {
           $agent_name = "Root";
           $AgentRootKey = "Software/VERITAS/VxSvc/CurrentVersion";
           `$REGCTL $REG_FILE openkey $AgentRootKey 2>>$LOG`;

            if($?)
            {
               clean_up("The VEA service (vxsvc) has not been installed on this machine\n");
            }
       }
       else
       {
           $AgentRootKey = "Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/".$agent_name;
           `$REGCTL $REG_FILE openkey $AgentRootKey 2>>$LOG`;
           if($?)
           {
               clean_up("The agent $agent_name has not been installed on this machine.\n");
           }

       }


       if ( exists($sg_conf{'PARALLEL'}) && ($sg_conf{'PARALLEL'} eq 1) )
       {
           #Copy the VeaAgent files to the VCS directories
           SetupVCSAgent();

           if($opts_s)
           {
               #Include VeaAgentTypes.cf in main.cf
               `hatype -display VeaAgent > /dev/null 2>&1`;
               if($?)
                {
                   `hatype -add VeaAgent 2>>$LOG`;
                   `haattr -add VeaAgent UserId -string 2>>$LOG`;
                   `haattr -add VeaAgent AgentName -string 2>>$LOG`;
                   `hatype -modify VeaAgent ArgList -add AgentName UserId 2>>$LOG`;
                    ExecuteCmd("hatype -modify VeaAgent SourceFile \"/etc/VRTSvcs/conf/config/VeaAgentTypes.cf\"",
                    "ERROR: failed to create the VeaAgent type file VeaAgentTypes.cf\n");
                }

                #Add VeaAgent resource

                my $ag_res=$sg_conf{'GRP_NAME'}."_".$agent_name;

                ExecuteCmd("hares -add $ag_res  VeaAgent $sg_conf{'GRP_NAME'}",
                "ERROR: Failed to add  VeaAgent resoruce for $agent_name\n");

                vcs_log("hares -delete $ag_res");

                ExecuteCmd("hares -modify $ag_res AgentName $agent_name",
                "ERROR: Failed to add AgentName attribute to VeaAgent resoruce for $agent_name \n");
           }
           return;
       }



       `/opt/VRTSobc/$PAL_VERSION/bin/vxpal -a $agent_name -c status`;
       if(!$?)
       {
           if($agent_name eq "Root")
           {
               clean_up("vxsvc is running on this machine.Please stop it.\n");
           }
           else
           {
               clean_up("The agent $agent_name is running on this machine.Please stop it.\n"); 
           }
       }   

       #Copy the VeaAgent files to the VCS directories
       SetupVCSAgent();

       #If master then do VCS setup for the agent 
       if ($opts_s)
       {

           my $ip_res = get_ip_res($conf{$agent_info}->{'VIRTUAL_HOSTNAME'});
           my $mnt_res = get_mnt_res($conf{$agent_info}->{'MOUNT_POINT'});
      
           #Include VeaAgentTypes.cf in main.cf
           `hatype -display VeaAgent > /dev/null 2>&1`;
            if($?)
            {
               `hatype -add VeaAgent 2>>$LOG`;
               `haattr -add VeaAgent UserId -string 2>>$LOG`;
               `haattr -add VeaAgent AgentName -string 2>>$LOG`;
               `hatype -modify VeaAgent ArgList -add AgentName UserId 2>>$LOG`;
                ExecuteCmd("hatype -modify VeaAgent SourceFile \"/etc/VRTSvcs/conf/config/VeaAgentTypes.cf\"",
                "ERROR: failed to create the VeaAgent type file VeaAgentTypes.cf\n");
            }
            #Add VeaAgent resource

            my $ag_res=$sg_conf{'GRP_NAME'}."_".$agent_name;

            ExecuteCmd("hares -add $ag_res  VeaAgent $sg_conf{'GRP_NAME'}",
            "ERROR: Failed to add  VeaAgent resoruce for $agent_name\n");

            vcs_log("hares -delete $ag_res");

            ExecuteCmd("hares -modify $ag_res AgentName $agent_name",
            "ERROR: Failed to add AgentName attribute to VeaAgent resoruce for $agent_name \n");

            #Create dependancy of the VeaAgent Resource on the IP resource
            ExecuteCmd("hares -link  $ag_res $ip_res",
            "ERROR: failed to add dependancy of VeaAgent resource for $agent_name on the IP resource $ip_res\n");

            vcs_log("hares -unlink  $ag_res $ip_res");

            #Create dependancy of the VeaAgent Resource on the mount resource
            ExecuteCmd("hares -link  $ag_res $mnt_res",
            "ERROR: failed to add dependancy of VeaAgent resource for $agent_name on the Mount resource $mnt_res\n");

            vcs_log("hares -unlink  $ag_res $mnt_res");
 
            #Create dependancy of the VeaAgent Resource on any other resources if specified
            if( exists($conf{$agent_info}{'DEPENDS'}))
            { 
                my @resources = split(/,/,$conf{$agent_info}{'DEPENDS'});

                foreach my $res (@resources)
                {
                    chomp($res);
                    ExecuteCmd("hares -link $ag_res $res",
                    "ERROR: failed to add dependancy of VeaAgent resource for $agent_name on resource $res Please verify that resource $res exits in the service group $sg_conf{'GRP_NAME'}");    
 
                    vcs_log("hares -unlink $ag_res $res"); 
                }

            }

            #If agent is CS then create the dependancy of vxsvc agent on ccsfdatabase
            # vxsvc->ccsf_database

            if ($conf{$agent_info}->{'NAME'} eq "CS") 
            {
                my $db_res = $sg_conf{'GRP_NAME'}."_ccsfdb";
                ExecuteCmd("hares -link  $ag_res $db_res",
                "ERROR: failed to create dependancy of VeaAgent resource for CS on the resource for ccsfdb");

                vcs_log("hares -unlink  $ag_res $db_res");

                $cs_configured = 1;                 
            }
           log_msg("VCS setup done for agent $agent_name\n");  

           log_msg("Doing cluster setup for Master for agent $agent_name\n");

           #Create a directory for this agent at the mount point
           if ( ! -d $shared_dir )
           {
               `mkdir -p $shared_dir`;
               if($?) {clean_up("ERROR: failed to share data\n");}
               
           }

           if ( $agent_name eq "Root" )
           {
               ShareFile($shared_dir,"AgentList", $AgentRootKey."/SecuredAgents");
               ShareFile($shared_dir,"alertlogfile",$AgentRootKey."/Providers/logger");
               ShareFile($shared_dir,"tasklogfile",$AgentRootKey."/Providers/logger");
  
               #DC name in registry and csf_resolv.conf are updated for different configurations 
               #as follows: 
               #1. For a non CPI install of VRTSob update DomainController when clusterize.pl is called 
               #    with agent name is non "CS"
               #2. For SFMS this is done as part of the script clusterize_sfms.pl during MH cluster configuration
               #3. For CMH it is expected that the MH are installed and configured to be part of the same domain 
               #   before it is clusterized.
               #4. This is also true for non CPI installation of VRTSobc package. 

               if( $conf{$agent_info}->{'NAME'} ne "CS" )
               {
                   my  $dc_key = "Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/__defaults/Network/Params/Security";
	          `$REGCTL $REG_FILE  setvalue $dc_key  DomainController REG_SZ $virtual_name  2>>$LOG`;
            
                   #Update the DC name in csf_resolv.conf
                   `$VXINI  -f $CSF_RESOLV_FILE -i -k domain -v $virtual_name -s DOMAIN_CONTROLLER 2>>$LOG`;
               } 
           }
           else
           {
               UpdateRegistrationInfo($agent_info,$AgentRootKey);
               ShareFile($shared_dir,"ManagerList", $AgentRootKey."/SecuredManagers");
           }

           ShareFile($shared_dir,"ObjectTypeFile",$AgentRootKey);
           ShareFile($shared_dir,"ObjectTypeDTD",$AgentRootKey);
           ShareFile($shared_dir,"InterfaceFormFile",$AgentRootKey);
           ShareFile($shared_dir,"InterfaceFormDTD",$AgentRootKey);
           ShareFile($shared_dir,"SecurityFormFile",$AgentRootKey);
           ShareFile($shared_dir,"SecurityFormDTD",$AgentRootKey);
           ShareFile($shared_dir,"AlertTypeFile",$AgentRootKey);
           ShareFile($shared_dir,"AlertTypeDTD",$AgentRootKey);

           #Set the virtual name in the registry for the agent
           `$REGCTL $REG_FILE  setvalue $AgentRootKey VirtualHostName REG_SZ $virtual_name 2>>$LOG`;

           #Set the persistent id for the agent
           `$REGCTL $REG_FILE  setvalue $AgentRootKey PERSISTENT_ID REG_SZ $pid 2>>$LOG`;


           if ($agent_name eq "gridnode")
           {
               ShareFile($shared_dir,"persistdir",$AgentRootKey."/Providers/gridnode"); 
           }
           elsif ($agent_name eq "gridcentral")
           {
               ShareFile($shared_dir,"persistdir",$AgentRootKey."/Providers/gridcentral");
           }
           elsif ($agent_name eq "actionagent")
           {
               ShareFile($shared_dir,"RULES_DB",$AgentRootKey."/Providers/actionagent");
               ShareFile($shared_dir,"EMAIL_CONF",$AgentRootKey."/Providers/actionagent");
           }

      }
      else
      {
           #For the files present on the shared disk modify the registry entries to
           #point to them

           if ( $agent_name eq "Root" )
           {
               SetRegVal($shared_dir,"AgentList", $AgentRootKey."/SecuredAgents");
               SetRegVal($shared_dir,"alertlogfile",$AgentRootKey."/Providers/logger");
               SetRegVal($shared_dir,"tasklogfile",$AgentRootKey."/Providers/logger");

               if( $conf{$agent_info}->{'NAME'} ne "CS" )
               {
                   my  $dc_key = "Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/__defaults/Network/Params/Security";
                  `$REGCTL $REG_FILE  setvalue $dc_key  DomainController REG_SZ $virtual_name  2>>$LOG`;

                   #Update the DC name in csf_resolv.conf
                   `$VXINI  -f $CSF_RESOLV_FILE -i -k domain -v $virtual_name -s DOMAIN_CONTROLLER 2>>$LOG`;
               }

           }
           else
           {
               UpdateRegistrationInfo($agent_info,$AgentRootKey);
               SetRegVal($shared_dir,"ManagerList", $AgentRootKey."/SecuredManagers");
           }

           SetRegVal($shared_dir,"ObjectTypeFile",$AgentRootKey);
           SetRegVal($shared_dir,"ObjectTypeDTD",$AgentRootKey);
           SetRegVal($shared_dir,"InterfaceFormFile",$AgentRootKey);
           SetRegVal($shared_dir,"InterfaceFormDTD",$AgentRootKey);
           SetRegVal($shared_dir,"SecurityFormFile",$AgentRootKey);
           SetRegVal($shared_dir,"SecurityFormDTD",$AgentRootKey);
           SetRegVal($shared_dir,"AlertTypeFile",$AgentRootKey);
           SetRegVal($shared_dir,"AlertTypeDTD",$AgentRootKey);

           #Set the virtual name in the registry for the agent
           `$REGCTL $REG_FILE  setvalue $AgentRootKey VirtualHostName REG_SZ $virtual_name 2>>$LOG`;

           #Set the persistent id for the agent
           `$REGCTL $REG_FILE  setvalue $AgentRootKey PERSISTENT_ID REG_SZ $pid 2>>$LOG`;

           if ($agent_name eq "gridnode")
           {
               SetRegVal($shared_dir,"persistdir",$AgentRootKey."/Providers/gridnode");
           }
           elsif ($agent_name eq "gridcentral")
           {
               SetRegVal($shared_dir,"persistdir",$AgentRootKey."/Providers/gridcentral");
           }
           elsif ($agent_name eq "actionagent")
           {
               SetRegVal($shared_dir,"RULES_DB",$AgentRootKey."/Providers/actionagent");
               SetRegVal($shared_dir,"EMAIL_CONF",$AgentRootKey."/Providers/actionagent");
           }
      }

      log_msg("VEA specific setup done for agent $agent_name\n");
      close(REGFD);

  }
}

sub Usage
{
    print_msg (INFO,"Usage: \n");
    print_msg (INFO,"clusterize.pl -h --help | -c --conf <conf_file> [-p --password <password> ] [-s --setup] \n");
}

sub LongUsage
{

    Usage();
    print_msg(INFO,"\nOptions:\n"
    ." [-h --help]                Help. shows this message.\n"
    ." [-p --password <password>] user's password.[password for the user specified in the configuration file]\n"
    ." [-c --conf <conf_file>]    The Cluster configuration file.\n"
    ."                            Template for the config file is /opt/VRTSobc/pal33/etc/cluster.conf\n"
    ." [-s --setup ]              Does cluster configuration for the agent(s) specified in the config file on the master.\n"
    ."\n                            Please ensure the following while using this option:\n"
    ."                            1. VCS engine should be up on the host.\n"
    ."                            2. The domain controller (if any) to which the agent(s) has registered with are up.\n"
    ."                            3. There is a shared device mounted at the mount point specified in the AGENT section.\n"
    ."                            4. The agent(s) specified in the config file are not running on the host.\n"	
    ."\n                            Do not specify this option for cluster configuration on the slave.\n"
    ."                            Please ensure the following during cluster configuration on the slave:\n"
    ."                            1. It is recommended that VCS engine is not up on the slave host\n"
    ."                            2. The domain controller (if any) to which the agent(s) has registered with are up.\n"
    ."                            3. The agent(s) specified in the config file are not running on the host.\n");
 
}


sub pwread {
        $| = 1;
        print "";
        my $what;
        my $rstring;
        my $lflags;

        my $numfile = fileno(STDIN);
        my $keyboard = new POSIX::Termios ();
        $keyboard -> getattr($numfile);

        # switch to cbreak mode

        my $cbset = $lflags = $keyboard -> getlflag();
        $cbset &= ~(ECHO|ECHOK|ICANON);
        $keyboard -> setlflag($cbset);
        $keyboard -> setcc(VTIME,1);
        $keyboard -> setattr($numfile,TCSANOW);

        until (0) {
                sysread(STDIN,$what,1);
                last if ($what =~ /[\n\r]/);
                if ($what =~ /[\x7f\x08]/) {
                        chop $rstring &&
                        print "\x08 \x08";
                } else {
                        $rstring .= $what;
                        }
                }

        # restore cooked mode

        $keyboard -> setlflag($lflags);
        $keyboard -> setcc(VTIME,0);
        $keyboard -> setattr($numfile,TCSANOW);

        print "\n";
        $| = 0;
        $rstring;
}



#Validate config file
sub ValidateConfFile()
{
    #If conf file not specified pick it up from the environment variable 
    if( $conf_file eq "")
    {
        $_ = $ENV{"VEA_CLUSTER_CONF"};
        chomp;
        $conf_file = $_;
        if( $conf_file eq "" )
        {
            print_msg(ERR,"Cluster config file not specified.\n"
            ."Specify it through -c option or set the environment variable VEA_CLUSTER_CONF.\n");
            Usage();
            exit(1);
        }
    }
    if ( ! -f $conf_file )
    {
        clean_up("The cluster config file $conf_file does not exist.Please specify a valid config file.\n");
    }

    log_msg("Using $conf_file as the cluster config file\n");

    #Validate the config file
    `$VXINI -f $conf_file -q 2>> $LOG`;
    if($?)
    {
        clean_up("ERROR: Config file $conf_file contains syntax error(s).\n");
    }

    if ($opts_s)
    {
        #If Master then service group section has to be present
        `$VXINI -f  $conf_file -q -s SERVICE_GROUP 2>> $LOG`;
        if($?)
        {
            clean_up("ERROR: service group not specified in the config file $conf_file.\n");
        }
     
      #TO DOIf Master then check if VCS is up

    }
    log_msg("Validated the cluster config file $conf_file\n");
}

sub ExecuteCmd
{
    my $error_str =  pop(@_);
    my $cmd = pop(@_);
    my $ret;

    $_ = `$cmd 2>> $LOG`;

    if($?)
    {
        clean_up($error_str); 
    }
    else
    {
        chomp;
        $ret = $_;
        return $ret;   
    } 

}

#Read the AGENT info from the conf file into a hash .
sub ParseAgents
{
   `$VXINI -f $conf_file -q | grep AGENT > $temp`;
   open(H,"$temp");
   while(<H>)
   {
     chomp;
     my $agent_sec = $_;

     #Get the agent name  
     $_ = ExecuteCmd("$VXINI -f $conf_file -q -s $agent_sec -k NAME",
     "ERROR: Agent name not specified under section $agent_sec\n");
 
     my $agent = $_; 
     $conf{$agent}{'NAME'} = $agent;

     if ( exists($sg_conf{'PARALLEL'}) && ($sg_conf{'PARALLEL'} eq 1) )
     {
        #If agent is a part of a parallel service group 
        #then rest of the keys are not needed.
         return ;
     }
 
     #Get the virtual host name 
     $conf{$agent}{'VIRTUAL_HOSTNAME'} = ExecuteCmd("$VXINI -f $conf_file -q -s $agent_sec -k VIRTUAL_HOSTNAME",
     "ERROR: VIRTUAL_HOSTNAME key not specified in the section $agent_sec in the config file $conf_file \n");

     #Get the agent PID
     $conf{$agent}{'PID'} = ExecuteCmd("$VXINI -f $conf_file -q -s $agent_sec -k PID",
     "ERROR: PID Key not specified in the section $agent_sec in the config file $conf_file\n");
 
     #Get the MOUNT POINT.
     $conf{$agent}{'MOUNT_POINT'} = ExecuteCmd("$VXINI -f $conf_file -q -s $agent_sec -k MOUNT_POINT",
     "ERROR: MOUNT_POINT key not specified in the section $agent_sec in the config file $conf_file\n");     


     #If master then check if MOUNT,IP AND NIC sections are present
     if($opts_s)
     {
         if ( ! -d $conf{$agent}{'MOUNT_POINT'} )
         {
             clean_up("The mount point $conf{$agent}{'MOUNT_POINT'} does not exist.please specify a valid mount point\n");
         }

         $conf{$agent}{'DEPENDS'} = `$VXINI -f $conf_file -q -s $agent_sec -k DEPENDS 2>/dev/null`;
      }
      log_msg("Parsed the AGENT section for the agent $agent successfully\n");
  }
  close(H);

}

sub ShareFile
{
   my $Key =  pop(@_);
   my $Attr =  pop(@_);
   my $dir = pop(@_);

   $_ = `$REGCTL $REG_FILE queryvalue $Key $Attr 2>>$LOG`;

   if ( $_ ne "" )
   {
      chomp;
      split(/: /);

      my $val = $_[1];
      my $new_dir;
      my $new_val;

      #If registry entry already points to the shared device then nothing needs to be done
      if( $val =~ m/^$dir/ )
      {
          return;

      } 
      if( -d $val)
      {
          $new_dir = $dir."/".$val;
          if( ! -d $new_dir)
          {
              `mkdir -p $new_dir 2>>$LOG`;
          } 
          `cp -Rf $val/* $new_dir 2>>$LOG`;
      }
      else
      {
         my($sub_dir,$filename) = $val =~ m/(.*\/)(.*)$/;
         $new_dir = $dir."/".$sub_dir;
         if( ! -d  $new_dir)
         {
              `mkdir -p $new_dir 2>>$LOG`;
         }   
         `cp $val $new_dir 2>>$LOG`;
      }

      print REGFD "$Key $Attr REG_SZ $val\n";

      #Set the path in the registry as the shared directory path
      $new_val = $dir."/".$val;
      `$REGCTL $REG_FILE setvalue $Key $Attr REG_SZ $new_val 2>>$LOG`;
   }
}

sub SetRegVal
{
   my $Key =  pop(@_);
   my $Attr =  pop(@_);
   my $dir = pop(@_);

   $_ = `$REGCTL $REG_FILE queryvalue $Key $Attr 2>>$LOG`;
  
   if ( $_ ne "" )
   { 
      chomp;
      split(/: /);

      my $val = $_[1];
      my $new_val;

      #If registry entry already points to the shared device then nothing needs to be done
      if( $val =~ m/^$dir/ )
      {
          return;
      }

      #Save the original registry entry to a file
      print REGFD "$Key $Attr REG_SZ $val\n";

      $new_val = $dir."/".$val;
      #Set the path in the registry as the shared directory path
      `$REGCTL $REG_FILE setvalue $Key $Attr REG_SZ $new_val`
  }
}

#clean up in case of cluster set up failure.
sub clean_up
{
    my $msg = pop(@_);
    print_msg(ERR,"$msg"
    ."The cluster set up failed.\n");

    log_msg("$msg");    
    log_msg("The cluster set up failed.");

    close(REGFD);
    close(VCSFD);

    #Revert back ne registry changes
    open(H,"$REG_BCK");
    while(<H>)
    {
        chomp;
       `$REGCTL $REG_FILE setvalue $_`;
    }
    close(H);

    `rm $REG_BCK  2>>$LOG`;
    `rm $temp 2>>$LOG`;
    `rm $temp2 2>>$LOG`;

    #revert back any changes make to VCS config file 
    open(H,"$VCS_LOG");
    while(<H>)
    {
       chomp;
       my @cmd_list = split(/;/,$_);
       my @list = reverse(@cmd_list);
       foreach my $cmd (@list)
       {
           chomp($cmd);
           log_msg("$cmd");
           `$cmd 2>>$LOG`;
       }
    }
    close(H);

   `rm $VCS_LOG 2>>$LOG`;

    log_msg("clean up done .\n"); 
    exit(1);
}

sub exit_success
{
    print_msg(INFO,"Cluster setup successful.\n");

    `rm $REG_BCK 2>>$LOG`;
    `rm $temp 2>>$LOG`;
    `rm $temp2 2>>$LOG`;
    `rm $VCS_LOG 2>>$LOG`;

    log_msg("Cluster setup successful\n");
    exit(0);
}


sub SetupVCSAgent
{
   #create the VCS Agent scripts if not already created. 
   my $VCSAgentDest = "$VCS_HOME/bin/VeaAgent";
   if( ! -d $VCSAgentDest )
   {  
      `mkdir $VCSAgentDest`;
      `cp /opt/VRTSvcs/bin/ScriptAgent $VCSAgentDest/VeaAgentAgent `;
      `cp $VCSAgentPath/VCSAgentMonitor $VCSAgentDest/monitor `;
      `cp $VCSAgentPath/VCSAgentOnline $VCSAgentDest/online `;
      `cp $VCSAgentPath/VCSAgentOffline $VCSAgentDest/offline `;
      `cp $VCSAgentPath/VCSAgentClean $VCSAgentDest/clean `;
      `cp $VCSAgentPath/VCSAgentTypes.cf $VCS_CONF/conf/config/VeaAgentTypes.cf `;

   }
 
   log_msg("Created files for VeaAgent in VCS\n");
}

sub UpdateRegistrationInfo
{
    my $agent_info = shift;
    my $AgentRootKey = shift;

    my $managerlist;
    my $HC;
    my $LocalHost;
    my $DC;

    my $agent_name = $conf{$agent_info}->{'NAME'};
    my $virtual_name = $conf{$agent_info}->{'VIRTUAL_HOSTNAME'};
    my $pid = $conf{$agent_info}->{'PID'};


    #For each domain controller the agent has registered with , leave the domain and
    #if the configuration is for master than rejoin the domain with the virtual name so that the
    #domain.cf of the domain controller reflects the virtual name and the specified persistent id of the agent.

    my $user_admin = "admin";
    my $sec_key = "Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/__defaults/Network/Params/Security";

    #Get the Manager list
    my $key = $AgentRootKey."/SecuredManagers";
    $_ = `$REGCTL $REG_FILE  queryvalue $key ManagerList 2>>$LOG`;
    if ( $_ ne "" )
    {
        chomp;
        split(/:/);

        #Set the path in the registry as the shared directory path
        $managerlist = $_[1];
    }
    else
    {
        #The manager list need not be present in case the agent is standalone.
        log_msg("domain.cf not present for the agent $agent_name.\n");
        return;
    }

    #If the agent not registered with any domain nothing needs to be done now.

    $HC = 1;
    open(H,"$managerlist");
    while(<H>)
    {
        chomp;
        split(/{/);
        $DC = $_[0];
        if( $DC ne "" )
        {
           $HC = 0;
           last;
        }
    }

    close (H);

    if ( $HC eq 1 )
    {
        log_msg("Agent $agent_name is not registered with any domain\n");
        return;
    }


    #Get The local host name 
    $_ = `$REGCTL $REG_FILE  queryvalue $sec_key LocalSystem  2>/dev/null`;
    if( $_ ne "")
    {
        chomp;
        split(/: /);
        $LocalHost = $_[1];
    }
    else
    {
        clean_up("ERROR:LocalSystem key is missing from the registry.\n");
    }

    #get the agent account from the Registry
    if ( $agent_name eq "Root" )
    {
        $sec_key = "Software/VERITAS/VxSvc/CurrentVersion/Network/Params/Security";
    }
    else
    {
        $sec_key = "Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/$agent_name/Network/Params/Security";
    }

    my $domain_name;
    my $domain_type;
    my $prpl_name;

    if( !exists($sg_conf{'USER_ID'}) && !read_key($sec_key,"DomainName",$domain_name))
    {
        if(!read_key($sec_key,"DomainType",$domain_type))
        {
           if(!read_key($sec_key,"SecurityId",$prpl_name))
           {
              $sg_conf{'USER_ID'} = "$prpl_name\@$domain_name.$domain_type";
           }
        }   
    }

    my $cmd;
    open (H, "$managerlist");
    while (<H>)
    {
        chomp;
        split(/{/);
        $DC = $_[0];

        log_msg("Agent $agent_name leaving domain $DC");

        $cmd = "$VEACONFIG -c leavedc -l $DC -a $agent_name\@$LocalHost ";
        if( exists($sg_conf{'USER_ID'}) )
        {
            $cmd = $cmd." -u $sg_conf{'USER_ID'} ";
        }
        if($passwd)
        {  
            $cmd = $cmd." -p $passwd";
        }

        log_msg($cmd);
        `$cmd`;

        if($?)
        {
            clean_up("ERROR: Failed to leave the domain $DC.\n");
        }

         #Set the virtual name in the registry for the agent
        `$REGCTL $REG_FILE setvalue $AgentRootKey VirtualHostName REG_SZ $virtual_name 2>>$LOG`;

         #Set the persistent id for the agent
        `$REGCTL $REG_FILE  setvalue $AgentRootKey PERSISTENT_ID REG_SZ $pid 2>>$LOG`;
      
        if($opts_s)
        {
            log_msg("Agent $agent_name joining domain $DC with virtual hostname $virtual_name");

            $cmd = "$VEACONFIG -c joindc -l $DC -a $agent_name\@$LocalHost ";
            if( exists($sg_conf{'USER_ID'}) )
            {
                $cmd = $cmd." -u $sg_conf{'USER_ID'} ";
            }
            if($passwd)
            {
                $cmd = $cmd." -p $passwd";
            }

            log_msg($cmd);
            `$cmd`;

            if($?)
            {
                clean_up("ERROR: Failed to join the domain $DC.\n");
            }
        }
     }
     close (H);
     log_msg("Updated $agent_name's registration information\n");
}

sub configure_service_grp
{
  `hagrp -display $sg_conf{'GRP_NAME'} >/dev/null 2>&1 `;
   if($?)
   {
      #Add the specified service group to main.cf
      ExecuteCmd("hagrp -add $sg_conf{'GRP_NAME'}",
       "ERROR: Failed to add service group\n");

      vcs_log("hagrp -delete $sg_conf{'GRP_NAME'}");

      #Add the SystemList
      ExecuteCmd("hagrp -modify $sg_conf{'GRP_NAME'} SystemList -add $sg_conf{'SYSTEM_LIST'}",
      "ERROR: Failed to add SystemList \n");

      #Add the AutoStartList
      ExecuteCmd("hagrp -modify $sg_conf{'GRP_NAME'} AutoStartList $sg_conf{'AUTO_START_LIST'}",
      "ERROR: Failed to add AutoStartList \n");

      `$VXINI -f $conf_file -q -s SERVICE_GROUP -k PARALLEL`;
      if (!$?)
      {
          ExecuteCmd("hagrp -modify $sg_conf{'GRP_NAME'} Parallel $sg_conf{'PARALLEL'}",
          "ERROR: Failed to set the Parallel attribute for the service group\n");
      }

      log_msg("Service group $sg_conf{'GRP_NAME'} configured \n");
   }
}

sub configure_nic
{

    #If NIC section present then create a NIC resource 
    `$VXINI -f $conf_file -q -s NIC 2>> $LOG `;

    if(!$?)
    {
       ExecuteCmd("hares -add $sg_conf{'GRP_NAME'}_NIC NIC $sg_conf{'GRP_NAME'}",
       "ERROR: Failed to add NIC resource\n"); 

       vcs_log("hares -delete $sg_conf{'GRP_NAME'}_NIC");

       #Add the device name for the NIC resource
       ExecuteCmd("hares -local $sg_conf{'GRP_NAME'}_NIC Device",
       "ERROR: Failed to make Device attribute local\n");

       while ( my ($key, $value) = each(%nic_conf) )
       {
           ExecuteCmd("hares -modify $sg_conf{'GRP_NAME'}_NIC Device $value  -sys $key`",
           "ERROR: Failed to add Device to NIC resource \n");
       }


       #Add optional parameters for NIC resource
       $_ = `$VXINI -f $conf_file -q -s NIC -k PING_OPTIMIZE `;
       if(!$?)
       {
          chomp;
          `hares -modify $sg_conf{'GRP_NAME'}_NIC PingOptimize $_`;
          if($?)
          {
              clean_up("ERROR: Failed to add PingOptimize attribute to the IP resource \n");
          }
       }

       $_ = `$VXINI -f $conf_file -q -s NIC -k MII `;
       if(!$?)
       {
            chomp;
           `hares -modify $sg_conf{'GRP_NAME'}_NIC Mii $_`;
           if($?)
           {
               clean_up("ERROR: Failed to add Mii \n");
           }
       }

       $_ = `$VXINI -f $conf_file -q -s NIC -k NETWORK_HOSTS `;
       if(!$?)
       {
           chomp;
           `hares -modify $sg_conf{'GRP_NAME'}_NIC NetworkHosts $_`;
           if($?)
           {
               clean_up("ERROR: Failed to add NetworkHosts \n");
           }
       }
       log_msg("configured $sg_conf{'GRP_NAME'}_NIC resource of type NIC in service group $sg_conf{'GRP_NAME'}\n");
   } 
}


sub configure_ip
{

    #If IP section present then create a IP resource
    `$VXINI -f $conf_file -q -s IP 2>> $LOG `;

    if(!$?)
    {
       ExecuteCmd("hares -add $sg_conf{'GRP_NAME'}_IP IP $sg_conf{'GRP_NAME'}",
       "ERROR: Failed to add IP resource\n");

       vcs_log("hares -delete $sg_conf{'GRP_NAME'}_IP");

       #Add the device name for the IP resource
       ExecuteCmd("hares -local $sg_conf{'GRP_NAME'}_IP Device`",
       "ERROR: Failed to make Device attribute local\n");

       while ( my ($key, $value) = each(%nic_conf) )
       {
          ExecuteCmd("hares -modify $sg_conf{'GRP_NAME'}_IP Device $value -sys $key`",
          "ERROR: Failed to add Device to IP resource\n");
       }

       #Add IP address
       ExecuteCmd("hares -modify $sg_conf{'GRP_NAME'}_IP Address $ip`",
        "ERROR: Failed to add IP Address\n");

       #Add NetMask if specified.
       $_ = `$VXINI -f $conf_file -q -s IP -k NET_MASK `;
       if (!$?)
       {
          chomp;
          `hares -modify $sg_conf{'GRP_NAME'}_IP NetMask $_`;
          if($?)
          {
              clean_up("ERROR: Failed to add NetMask \n");
          }
       }

       #Add ifconfig options
       $_ = `$VXINI -f $conf_file -q -s IP -k OPTIONS `;
       if (!$?)
       {
          chomp;
         `hares -modify $sg_conf{'GRP_NAME'}_IP Options $_`;
         if($?)
         {
             clean_up("ERROR: Failed to add Options \n");
         }
       }

       ExecuteCmd("hares -link $sg_conf{'GRP_NAME'}_IP $sg_conf{'GRP_NAME'}_NIC",
       "ERROR: Failed to create dependancy of IP resource on NIC resource\n"); 

       vcs_log("hares -unlink $sg_conf{'GRP_NAME'}_IP $sg_conf{'GRP_NAME'}_NIC");

       log_msg("configured $sg_conf{'GRP_NAME'}_IP resource of type IP in service group $sg_conf{'GRP_NAME'}\n");
  }

}

sub configure_mounts
{
  #Add Mount resources
  for my $mount_info ( keys %mount_conf )
  {
     my $mount_res = $sg_conf{'GRP_NAME'}."_".$mount_info;
     ExecuteCmd("hares -add $mount_res Mount $sg_conf{'GRP_NAME'}`",
     "ERROR: Failed to add Mount resource\n"); 

     vcs_log("hares -delete $mount_res");

     ExecuteCmd("hares -modify $mount_res MountPoint $mount_conf{$mount_info}->{'MOUNTPOINT'}",
     "ERROR: Failed to add MountPoint\n"); 

     ExecuteCmd("hares -modify $mount_res BlockDevice $mount_conf{$mount_info}->{'BLOCK_DEVICE'}",
     "ERROR: Failed to add BlockDevice\n");

     ExecuteCmd("hares -modify $mount_res FsckOpt $mount_conf{$mount_info}->{'FSCK_OPT'}",
     "ERROR: Failed to add FsckOpt\n"); 

     #Add the optional parameters
    `hares -modify $mount_res FSType $mount_conf{$mount_info}->{'FSTYPE'}`
       if exists $mount_conf{$mount_info}->{'FSTYPE'};

    `hares -modify $mount_res MountOpt $mount_conf{$mount_info}->{'MOUNT_OPT'}`
       if exists $mount_conf{$mount_info}->{'MOUNT_OPT'};

    `hares -modify $mount_res SnapUmount $mount_conf{$mount_info}->{'SNAP_UMOUNT'}`
       if exists $mount_conf{$mount_info}->{'SNAP_UMOUNT'};

     #Create dependancy of the Mount Resource on any other resources if specified
     if( exists($mount_conf{$mount_info}->{'DEPENDS'}) )
     {
          my @resources = split(/,/,$mount_conf{$mount_info}->{'DEPENDS'});

          foreach my $res (@resources)
          {
              chomp($res);
              ExecuteCmd("hares -link $mount_res $res",
              "ERROR: failed to add dependancy of Mount resource on  $res. Please verify that resource $res exits in the service group $sg_conf{'GRP_NAME'}");

              vcs_log("hares -unlink $mount_res $res");
          }
     }

     log_msg("configured $mount_res of type Mount resource in $sg_conf{'GRP_NAME'}\n");
  }

}

sub configure_webapp
{
    my $VEA_WEBAPP_DIR="/opt/VRTSob/webgui/vcs";

    #If VRTSWebAPP section present then create a resource for it in the VCS main.cf file 
    `$VXINI -f $conf_file -q -s VRTSWebAPP 2>> $LOG `;
    if(!$?)
    {
        my $web_res = $sg_conf{'GRP_NAME'}."_".$webapp_conf{'APP_NAME'};

        if ($webapp_conf{'APP_NAME'} eq "VEAWeb")
        {
             ExecuteCmd("hares -add $web_res Application $sg_conf{'GRP_NAME'}",
             "ERROR: Failed to add VRTSWebApp resource \n");

             vcs_log("hares -delete $web_res");

             ExecuteCmd("hares -modify $web_res User root",
             "ERROR: Failed to add User attribute to WebApp resource\n");

             ExecuteCmd("hares -modify $web_res StartProgram $VEA_WEBAPP_DIR/online.sh",
             "ERROR: Failed to add StartProgram attribute to WebApp resource\n");

             ExecuteCmd("hares -modify $web_res StopProgram $VEA_WEBAPP_DIR/offline.sh",
             "ERROR: Failed to add StopProgram attribute to WebApp resource\n");

             ExecuteCmd("hares -modify $web_res MonitorProgram $VEA_WEBAPP_DIR/monitor.sh",
             "ERROR: Failed to add  MonitorProgram attribute to WebApp resource\n");
        }
        else
        {
             ExecuteCmd("hares -add $web_res VRTSWebApp $sg_conf{'GRP_NAME'}",
             "ERROR: Failed to add VRTSWebApp resource \n");

             vcs_log("hares -delete $web_res"); 

             ExecuteCmd("hares -modify $web_res AppName $webapp_conf{'APP_NAME'}",
             "ERROR: Failed to add AppName  to VRTSWebApp resource \n");

             ExecuteCmd("hares -modify $web_res InstallDir $webapp_conf{'INSTALL_DIR'}",
             "ERROR: Failed to add InstallDir to VRTSWebApp resource \n"); 
       }    
 
       ExecuteCmd("hares -link $web_res $sg_conf{'GRP_NAME'}_IP",
       "ERROR: Failed to create dependancy of VRTSWebAPP resource on IP resource");  

       vcs_log("hares -unlink $web_res $sg_conf{'GRP_NAME'}_IP");
       log_msg("configured $web_res of type VRTSWebApp in $sg_conf{'GRP_NAME'}\n");
    } 

}


sub configure_asa_dbserver
{
    #If ASA_DB_SERVER section present then create a resource for it 
    `$VXINI -f $conf_file -q -s ASA_DB_SERVER 2>> $LOG `;
    if(!$?)
    {
        my $DB_START_PRG = "$DBS_SCRIPT_DIR/online.sh";
        my $DB_STOP_PRG = "$DBS_SCRIPT_DIR/offline.sh";
        my $DB_MONITOR_PRG = "$DBS_SCRIPT_DIR/monitor.sh";


        ExecuteCmd("hares -add $asa_db_server Application $sg_conf{'GRP_NAME'}",
         "ERROR: Failed to add resource for dbserver \n");

        vcs_log("hares -delete $asa_db_server");

        ExecuteCmd("hares -modify $asa_db_server User root",
         "ERROR: Failed to add User attribute to dbserver resource\n");

        ExecuteCmd("hares -modify $asa_db_server StartProgram $DB_START_PRG",
         "ERROR: Failed to add StartProgram attribute to dbserver resource\n");

        ExecuteCmd("hares -modify $asa_db_server StopProgram $DB_STOP_PRG",
         "ERROR: Failed to add StopProgram attribute to dbserver resource\n");

        ExecuteCmd("hares -modify $asa_db_server MonitorProgram $DB_MONITOR_PRG",
         "ERROR: Failed to add  MonitorProgram attribute to dbserver resource\n");

        log_msg("configured $asa_db_server of type Application in the service group $sg_conf{'GRP_NAME'}");
    }
}

sub configure_asa_database
{
    for my $db ( keys %asa_database )
    {
        if($opts_s)
        {
            my $db_path = $asa_database{$db}{'DB_PATH'};
            my($dir,$file) = $db_path =~ m/(.*\/)(.*)$/;
            chomp($dir);
        
            #If master then copy the database file to shared location
            if(!-d $dir)
            {
                ExecuteCmd("mkdir -p $dir","Faile to share database $asa_database{$db}{'DB_NAME'}");
            }

            $ENV{'ODBCINI'} = $asa_database{$db}{'ODBC_INI'};
            $ENV{'LD_LIBRARY_PATH'} = "/opt/VRTSdbms3/lib";

            my $con_str = `/opt/VRTSdbms3/bin/dbdsn  -g $asa_database{$db}{'DB_NAME'} -q`;
            chomp($con_str);
            my ($dbf,$rest) = $con_str =~ m/(DBF=[^;]*;)(.*)/;
            split(/=/,$dbf);
            $dbf = $_[1];
            chop($dbf);
            `cp $dbf $dir 1>/dev/null`;
        }

        #setup the odbcini file to reflect the new location of the database file
        setupini($asa_database{$db}{'DB_PATH'},$asa_database{$db}{'DB_NAME'},$asa_database{$db}{'ODBC_INI'});

        if($opts_s)
        {
            my $DB_START_PRG = "\"$DB_SCRIPT_DIR/online.sh $asa_database{$db}{'DB_PATH'} $asa_database{$db}{'DB_NAME'} $asa_database{$db}{'ODBC_INI'}\"";
            my $DB_STOP_PRG = "\"$DB_SCRIPT_DIR/offline.sh $asa_database{$db}{'DB_NAME'} $asa_database{$db}{'ODBC_INI'}\"";
            my $DB_MONITOR_PRG = "\"$DB_SCRIPT_DIR/monitor.sh $asa_database{$db}{'DB_NAME'} $asa_database{$db}{'ODBC_INI'}\"";

            my $asa_db_res = $sg_conf{'GRP_NAME'}."_".$asa_database{$db}{'DB_NAME'};
            ExecuteCmd("hares -add $asa_db_res Application $sg_conf{'GRP_NAME'}",
            "ERROR: Failed to add resource for database $asa_database{$db}{'DB_NAME'}\n");

            vcs_log("hares -delete $asa_db_res");

            ExecuteCmd("hares -modify $asa_db_res User root",
            "ERROR: Failed to add User attribute to database resource\n");

            ExecuteCmd("hares -modify $asa_db_res StartProgram $DB_START_PRG",
            "ERROR: Failed to add StartProgram attribute to database resource\n");

            ExecuteCmd("hares -modify $asa_db_res StopProgram $DB_STOP_PRG",
            "ERROR: Failed to add StopProgram attribute to database resource\n");

            ExecuteCmd("hares -modify $asa_db_res MonitorProgram $DB_MONITOR_PRG",
            "ERROR: Failed to add  MonitorProgram attribute to database resource\n");

            @db_mount_agents = split(/,/,$asa_database{$db}{'DEPENDS'});

            #Add dependancy of the database resource on its mount agents
            foreach my $section (@db_mount_agents)
            {
                ExecuteCmd("hares -link $asa_db_res $section",
                "ERROR: Failed to add $asa_db_res dependancy on $section\n"); 

                vcs_log("hares -unlink $asa_db_res $section"); 
            }

            log_msg("configured resource for $asa_database{$db}{'DB_NAME'} (ASA database) in $sg_conf{'GRP_NAME'}");
        }
   }
}

sub ParseWebApp
{
    #If VRTSWebAPP section present then parse its keys
    `$VXINI -f $conf_file -q -s VRTSWebAPP 2>> $LOG `;

    if(!$?)
    {
        $webapp_conf{'APP_NAME'} = ExecuteCmd("$VXINI -f $conf_file -q -s VRTSWebAPP -k APP_NAME",
        "ERROR: APP_NAME key not specified in the VRTSWebAPP section\n");

        $webapp_conf{'INSTALL_DIR'} = ExecuteCmd("$VXINI -f $conf_file -q -s VRTSWebAPP -k INSTALL_DIR",
        "ERROR: INSTALL_DIR key not specified in the VRTSWebAPP section\n");

        log_msg("Parsed the VRTSWebAPP successfully\n");
    } 
}

sub ParseASADBServer
{
    #Parse the ASA dbserver section if present

    `$VXINI -f $conf_file -q -s ASA_DB_SERVER 2>> $LOG `;
    if(!$?)
    {
        if (-d "/opt/VRTSdbms3")
        {
            $VXDBMS_BASEDIR = "/opt/VRTSdbms3/";
        }
        else
        {
            if (-d "/opt/VRTSdbmsd")
            {
                $VXDBMS_BASEDIR = "/opt/VRTSdbmsd/";
            }
            else
            {
               clean_up("ERROR : Parsing ASA_DB_SERVER section.VRTSdbms package not installed.\n");
            }
         }

        $asa_db_server = ExecuteCmd("$VXINI -f $conf_file -q -s ASA_DB_SERVER -k RESOURCE_NAME",
        "ERROR: RESOURCE_NAME key not specified in the ASA_DB_SERVER section in the config file $conf_file\n");

        log_msg("Parsed the ASA_DB_SERVER section successfully\n");
    }
}

sub ParseASADatabase
{
    #Parse the ASA database section if present
    my @asa_database_arr = `$VXINI -f $conf_file -q | grep ASA_DATABASE`;

    foreach my $db_sec (@asa_database_arr)
    {
        if (-d "/opt/VRTSdbms3")
        {
            $VXDBMS_BASEDIR = "/opt/VRTSdbms3/";
        }
        else
        {
            if (-d "/opt/VRTSdbmsd")
            {
                $VXDBMS_BASEDIR = "/opt/VRTSdbmsd/";
            }
            else
            {
               clean_up("ERROR : Parsing ASA_DATABASE section.VRTSdbms package not installed on this machine.\n");
            }
        }

        chomp($db_sec);

        my $db = ExecuteCmd("$VXINI -f $conf_file -q -s $db_sec -k DB_NAME",
        "ERROR: DB_NAME key not specified in the section $db_sec in the config file $conf_file\n");

        $asa_database{$db}{'DB_NAME'} = $db;

        $asa_database{$db}{'DB_PATH'} = ExecuteCmd("$VXINI -f $conf_file -q -s $db_sec -k DB_PATH",
         "ERROR: DB_PATH key not specified in the $db_sec section in the config file $conf_file\n");

        $asa_database{$db}{'DEPENDS'} = ExecuteCmd("$VXINI -f $conf_file -q -s $db_sec -k DEPENDS",
        "ERROR: DEPENDS key not specified in the $db_sec section\n");

        $asa_database{$db}{'ODBC_INI'} = ExecuteCmd("$VXINI -f $conf_file -q -s $db_sec -k ODBC_INI",
         "ERROR: ODBC_INI key not specified in the $db_sec section in the config file $conf_file\n");

        log_msg("Parsed the ASA_DATABASE section successfully\n");
    }
}

#Parse the SERVICE_GROUP section
sub ParseServiceGroup
{
    #Parse the service group section
    $sg_conf{'GRP_NAME'} = ExecuteCmd("$VXINI -f $conf_file -q -s SERVICE_GROUP -k GRP_NAME",
               "ERROR: Service group name not specified.\n");


    $_ = `$VXINI -f $conf_file -q -s SERVICE_GROUP -k USER_ID`;
    if(!$?)
    { 
         chomp;
         $sg_conf{'USER_ID'} = $_;
    }

    if($opts_s)
    {
        `hagrp -display $sg_conf{'GRP_NAME'} >/dev/null 2>&1`;
        if($?)
        {
            #If service group does not already exist then validate if SYSTEM_LIST 
            #and AUTO_START_LIST are present in the config file

            $sg_conf{'SYSTEM_LIST'} = ExecuteCmd("$VXINI -f $conf_file -q -s SERVICE_GROUP -k SYSTEM_LIST",
            "ERROR: SYSTEM_LIST key not specified in the SERVICE_GROUP section in the config file $conf_file. \n");

            $sg_conf{'AUTO_START_LIST'} = ExecuteCmd("$VXINI -f $conf_file -q -s SERVICE_GROUP -k AUTO_START_LIST",
            "ERROR: AUTO_START_LIST key not specified in the SERVICE_GROUP section in the config file $conf_file. \n");
        }
    }
    $_ = `$VXINI -f $conf_file -q -s SERVICE_GROUP -k PARALLEL `;
    if (!$?)
    {
       chomp;
       $sg_conf{'PARALLEL'} = $_;
    
    }
    log_msg("Parsed the SERVICE_GROUP section successfully\n");
}

   
#Parse the NIC section
sub ParseNIC
{
    `$VXINI -f $conf_file -q -s NIC > $temp2`;
    if(!$?)
    {
        my $nic;
        open(H,"$temp2");

        while(<H>)
        {
            chomp;
            my $sys = $_;

            $nic_conf{ $sys } = ExecuteCmd("$VXINI -f $conf_file -q -s NIC -k $sys",
            "ERROR: NIC device for $sys not specified.\n"); 

            $nic = 1;
        }
        close(H);

        if(!$nic)
        {
            clean_up("ERROR: Specify NIC device for each system in the SYSTEM_LIST.\n");
        }

        log_msg("Parsed the NIC section successfully\n");
    }
}

#Parse the IP section
sub ParseIP
{
    #Validate the IP section before editing main.cf
    `$VXINI -f $conf_file -q -s IP 2>> $LOG `;
    if (!$?)
    {
        $ip = ExecuteCmd("$VXINI -f $conf_file -q -s IP -k IP",
        "ERROR: IP key not present in the IP section.Remove the IP section is you dont want an IP resource ".
        "else specify the IP key\n");

        ExecuteCmd("$VXINI -f $conf_file -q -s NIC",
        "ERROR: IP section needs NIC section present in the config file $conf_file\n");
    }
}

#Parse the MOUNT* sections
sub ParseMOUNTS
{
  `$VXINI -f $conf_file -q | grep MOUNT > $temp2`;
   open(H,"$temp2");

   while(<H>)
   {
      chomp;
      my $mount = $_;

      #Parse the Required Keys
      $mount_conf{$mount}{'BLOCK_DEVICE'} = ExecuteCmd("$VXINI -f $conf_file -q -s $mount -k BLOCK_DEVICE",
      "ERROR: BLOCK_DEVICE not specified in the $mount section. \n");

      $mount_conf{$mount}{'FSCK_OPT'} =  ExecuteCmd("$VXINI -f $conf_file -q -s $mount -k FSCK_OPT",
      "ERROR: FSCK_OPT not specified in the $mount section. \n");

      $mount_conf{$mount}{'MOUNTPOINT'} = ExecuteCmd("$VXINI -f $conf_file -q -s $mount -k MOUNT_POINT",
       "ERROR: MOUNT_POINT not specified in the $mount section. \n");

      #Parse the optional keys

      $_ = `$VXINI -f $conf_file -q -s $mount -k FSTYPE `;
      if (!$?)
      {
         chomp;
         $mount_conf{$mount}{'FSTYPE'}= $_;
      }

      $_ = `$VXINI -f $conf_file -q -s $mount -k MOUNT_OPT `;
      if(!$?)
      {
         chomp;
         $mount_conf{$mount}{'MOUNT_OPT'}= $_;
      }

      $_ = `$VXINI -f $conf_file -q -s $mount -k SNAP_UMOUNT `;
      if(!$?)
      {
         chomp;
         $mount_conf{$mount}{'SNAP_UMOUNT'}= $_;
      }

      $_ = `$VXINI -f $conf_file -q -s $mount -k DEPENDS `;
      if(!$?)
      {
         chomp;
         $mount_conf{$mount}{'DEPENDS'}= $_;
      }

      log_msg("Parsed the MOUNT section $mount successfully\n");
   }
   close(H);
}

sub configure_disk_groups
{
    for my $dg_sec ( keys %disk_groups )
    {
        my $dg_res = $sg_conf{'GRP_NAME'}."_".$disk_groups{$dg_sec}{'DISK_GROUP'};

        ExecuteCmd("hares -add $dg_res DiskGroup $sg_conf{'GRP_NAME'}",
        "ERROR: Failed to add DiskGroup resource\n");

        vcs_log("hares -delete $dg_res");

        ExecuteCmd("hares -modify $dg_res DiskGroup $disk_groups{$dg_sec}{'DISK_GROUP'}",
        "ERROR: Failed to add DiskGroup attribute to resource $dg_res\n");

        `hares -modify $dg_res StartVolumes $disk_groups{$dg_sec}{'START_VOLUMES'}`
         if exists $disk_groups{$dg_sec}{'START_VOLUMES'};

        `hares -modify $dg_res StopVolumes  $disk_groups{$dg_sec}{'STOP_VOLUMES'}`
         if exists $disk_groups{$dg_sec}{'STOP_VOLUMES'};
    }
}

sub ParseDiskGroups
{
    #Parse DISK_GROUP sections
    my @dg_arr = `$VXINI -f $conf_file -q | grep DISK_GROUP`;

    foreach my $dg_sec (@dg_arr)
    {
       chomp($dg_sec);

       my $dg = ExecuteCmd("$VXINI -f $conf_file -q -s $dg_sec -k DISK_GROUP",
       "ERROR: DISK_GROUP key not specified under section $dg_sec in the config file $conf_file\n");

       $disk_groups{$dg}{'DISK_GROUP'} = $dg;

       $_ = `$VXINI -f $conf_file -q -s $dg_sec -k START_VOLUMES`;
       if(!$?)
       {
          chomp;
          $disk_groups{$dg}{'START_VOLUMES'} = $_;
       }

       $_ = `$VXINI -f $conf_file -q -s $dg_sec -k STOP_VOLUMES`;
       if(!$?)
       {
          chomp;
          $disk_groups{$dg}{'STOP_VOLUMES'} = $_;
       }
    }
}

sub configure_volumes
{
    for my $vol_sec ( keys %volumes )
    {
        my $vol_res = $sg_conf{'GRP_NAME'}."_".$volumes{$vol_sec}{'VOLUME'};

        ExecuteCmd("hares -add $vol_res Volume $sg_conf{'GRP_NAME'}",
        "ERROR: Failed to add Volume resource $vol_res\n");

        vcs_log("hares -delete $vol_res");

        ExecuteCmd("hares -modify $vol_res Volume $volumes{$vol_sec}{'VOLUME'}",
        "ERROR: Failed to add Volume attribute to Volume resource $vol_sec\n");

        ExecuteCmd("hares -modify $vol_res DiskGroup $volumes{$vol_sec}{'DISK_GROUP'}",
        "ERROR: Failed to add DiskGroup attribute to Volume resource $vol_sec\n");

        my $dg = $volumes{$vol_sec}{'DISK_GROUP'};
        my $dg_res = $sg_conf{'GRP_NAME'}."_".$dg;

        ExecuteCmd("hares -link $vol_res $dg_res",
        "ERROR: Failed to create dependancy of $vol_res on $dg_res");
   }
}


sub ParseVolumes
{
    #Parse VOLUME sections
    my @vols = `$VXINI -f $conf_file -q | grep VOLUME`;

    foreach my $vol_sec (@vols)
    {
       chomp($vol_sec);

       my $vol = ExecuteCmd("$VXINI -f $conf_file -q -s $vol_sec -k VOLUME",
       "ERROR: VOLUME key not specified under section $vol_sec in the config file $conf_file");

       $volumes{$vol}{'VOLUME'} = $vol;

       $volumes{$vol}{'DISK_GROUP'} =  ExecuteCmd("$VXINI -f $conf_file -q -s $vol_sec -k DISK_GROUP",
       "ERROR: DISK_GROUP key not specified under section $vol_sec in the config file $conf_file");

    }
}

sub log_msg
{
   $_ =  pop(@_);
   chomp;
   `echo $_ >> $LOG`;
}

sub vcs_log
{
   $_ =  pop(@_);
   chomp;
   print VCSFD "$_;";
}

sub get_ip_res
{
   my $host = pop(@_);
   my $ip = get_ip_address($host);
   my $type;
   my $temp;

   my @res = `hagrp -resources $sg_conf{'GRP_NAME'}`;
   foreach my $section (@res)
   {
      chomp($section);
      next if ($section eq ""); 
  
      $type = `hares -value $section Type`;
      chomp($type);
      if( $type eq "IP" )
      {
         $temp = `hares -value $section Address`;
         chomp($temp);
         if($ip eq $temp)
         {
             return $section;
         }
      }
   }
   clean_up("There is no IP resource configured for IP Address $ip \(corrosponding to $host\) \n"
   ."in the service group $sg_conf{'GRP_NAME'}.Please create an IP section in the config file for $ip\n");
}

sub get_mnt_res
{
   my $mnt_pt = pop(@_);

   my $type;
   #get the Mount resource for the mount point 

   my($base_dir,$rest) = $mnt_pt =~ m/(^\/[^\/]*)(.*)$/;

   chomp($base_dir);

   my @res = `hagrp -resources $sg_conf{'GRP_NAME'}`;
   foreach my $section (@res)
   {
      chomp($section);

      next if ($section eq "");
      $type = `hares -value $section Type`;
      chomp($type);

      if( $type eq "Mount" )
      {
         $temp = `hares -value $section MountPoint`;
         chomp($temp);

         my($temp_base_dir,$rest) = $temp =~ m/(^\/[^\/]*)(.*)$/;
         chomp($temp_base_dir);

         if($base_dir eq $temp_base_dir)
         {
             return $section;
         }
      }
   }
   clean_up("There is no Mount resource configured for the mount point $mnt_pt \n"
   ."in the service group $sg_conf{'GRP_NAME'}.Please create a MOUNT section in the\n"
   ."config file to create a Mount resource\n");

}


sub get_ip_address
{
   my $host = pop(@_);
   my $addr1 = gethostbyname($host);
   my $addr;

   if ( $addr1 ) {
      my($a,$b,$c,$d) = unpack ('C4', $addr1);
	$addr = "$a.$b.$c.$d";
   } else {
	clean_up( "IP Lookup failed for hostname $host.Please verify that the hostname is valid\n");
   }
   return $addr
}

sub setupini
{
    my $odbcini = pop(@_);
    my $db = pop(@_);
    my $db_file = pop(@_);

    my $ret = 0;
    my $servername;

    #replace the database file path in the odbcini file

    $ENV{'ODBCINI'} = $odbcini;

    my $DBMSDIR = "/opt/VRTSdbms3";
    my $lib_path = "$DBMSDIR/lib";
    $ENV{'LD_LIBRARY_PATH'} = $lib_path;

    my $con_str = `$DBMSDIR/bin/dbdsn -g $db  -q`;
    chomp($con_str);

    $con_str =~ s/DBF=[^;]*;/DBF=$db_file;/;
    split(/:/,$con_str);
    $con_str =$_[1];
    `$DBMSDIR/bin/dbdsn -w $db -c  "$con_str" -y`;

out:
    return $ret;
}

sub print_msg
{
    my $level = shift;
    my $msg = shift;

    
    if( ($level eq "ERROR") && ($error_level eq "ERROR"))
    {
        print STDERR "$level:$msg";
    }
    else
    {
        print STDOUT "$msg";
    } 
}

sub read_key
{
   my $key = shift;
   my $val = shift;

   my $ret_val = 0;

   my $val1 = `$REGCTL $REG_FILE queryvalue $key $val`;
   if(!$?)
   {
      chomp($val1);
      my @temp_arr = split(/: /,$val1);
      $_[0] = $temp_arr[1];
   }
   else
   {
      $ret_val = 1;
   }
   return $ret_val;
   
}

